package controller;

import static model.Ocean.BUBBLE;
import static model.Ocean.COKE_CAN;
import static model.Ocean.FISH;
import static model.Ocean.PLANT;
import static model.Ocean.SHARK;
import static model.Ocean.STONE;
import static model.Ocean.makeOOID;

import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import model.Bubble;
import model.CokeCan;
import model.Fish;
import model.Ocean.EmptyListException;
import model.Ocean.PositionOccupiedException;
import model.Ocean.PositionOutOfBoundsException;
import model.OceanInterface;
import model.OceanObject;
import model.Plant;
import model.Shark;
import model.Stone;
import view.AboutFrame;
import view.OceanGUIInterface;

/**
 * The listener interface for receiving button events. The class that is
 * interested in processing a button event implements this interface, and the
 * object created with that class is registered with a component using the
 * component's <code>addButtonListener<code> method. When
 * the button event occurs, that object's appropriate
 * method is invoked.
 *
 *
 */
public class ButtonListener extends Listener implements ActionListener {

	/**
	 * The Class EmptyStringException.
	 */
	private class EmptyStringException extends Exception {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = 7312146429306850592L;
	}

	/**
	 * This exception is thrown whenever there is excess white space in the text
	 * input fields.
	 */
	private class ExcessWhitespaceException extends Exception {
		private static final long serialVersionUID = 7312146429306850592L;
	}

	/**
	 * This exception is thrown whenever the maximum age in the maximum age
	 * field is negative.
	 */
	private class MaximumAgeNegativeException extends Exception {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = 7312146429306850592L;
	}

	/**
	 * Instantiates a new button listener.
	 *
	 * @param model
	 *            the model
	 * @param view
	 *            the view
	 */
	public ButtonListener(OceanInterface model, OceanGUIInterface view) {

		super(model, view);
	}

	/**
	 * This method gets an ActionEvent (with button was pressed) and execute the
	 * specific code.
	 *
	 * @param e
	 *            the action event that provides the info which button was
	 *            pressed.
	 *
	 */
	@Override
	public void actionPerformed(ActionEvent e) {

		switch (e.getActionCommand()) {

		case "Add":
			Boolean inputOK = true;
			String type = null;
			String name = null;
			int[] position = new int[2];
			int maximumAge = -1;

			// collect all data
			type = getTypeOfObject();
			name = getName();

			if (name == null) {
				inputOK = false;
			}

			if (inputOK) {

				maximumAge = getMaximumAge();
				if (maximumAge == -1) {
					inputOK = false;
				}
			}

			if (inputOK) {

				position[0] = getXPosition();
				if (position[0] == -1) {
					inputOK = false;
				}
			}

			if (inputOK) {

				position[1] = getYPosition();
				if (position[1] == -1) {
					inputOK = false;
				}
			}

			if (!inputOK)
				return;

			try {
				createObject(type, name, maximumAge, position);
				updateGUI();
			}

			catch (NumberFormatException e1) {

				message("Object can not be added to Ocean");
			}

			catch (PositionOccupiedException e2) {
				message("This position is already occupied.");
			}

			catch (PositionOutOfBoundsException e3) {

				message("The coordinates (" + position[0] + ", " + position[1]
						+ ") would place part or all of the "
						+ "object's image outside the ocean.\n"
						+ "The ocean is " + getMyOcean().getWidth()
						+ " pixels wide and " + getMyOcean().getDepth()
						+ " pixels deep.");
			}

			catch (Exception e4) {
				e4.printStackTrace();
			}
			updateGUI();
			break;

		case "Remove":

			try {
				getMyOcean().removeAllTaggedObjects();
			} catch (EmptyListException e3) {
				message("Nothing to remove");
			}
			updateGUI();
			break;

		case "Clear":

			getMyOcean().removeAllObjects();
			updateGUI();
			break;

		case "Start":
			getMyOceanGUI().getButtonPanel().getStartStopButton()
			.setText("Stop");
			getMyOceanGUI().getButtonPanel().getStepButton().setEnabled(false);
			getMyOcean().start();
			break;

		case "Step":

			try {
				getMyOcean().move();
			} catch (Exception e2) {
				message("Moving the ocean failed");
			}
			updateGUI();
			printBubbleThreadStates();
			break;

		case "Stop":
			getMyOcean().stop();
			getMyOceanGUI().getButtonPanel().getStartStopButton()
			.setText("Start");
			getMyOceanGUI().getButtonPanel().getStepButton().setEnabled(true);

			try {
				Thread.sleep(100);
			} catch (Exception e5) {System.err.println("Sleeping failed");}

			try {
				Thread.sleep(100);
			} catch (Exception e5) {System.err.println("Sleeping failed");}
			break;

		case "about":

			AboutFrame abFrame = new AboutFrame();
			abFrame.setVisible(true);
			break;

		case "Print":

			if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
				// wenn die Shift-Taste gedrückt ist
				showOverview();

			} else {

				System.out.println(getMyOcean());

			}
			break;

		case "Save":

			if (getMyOcean().isOceanRunning()) {
				getMyOceanGUI().getButtonPanel().getStartStopButton().doClick();
			}

			File nameSave = null;
			String pathSave = null;

			do {

				FileDialog helperFrame = new FileDialog(
						(Frame) getMyOceanGUI(), "Save", FileDialog.SAVE);

				helperFrame.setVisible(true);

				try {
					pathSave = helperFrame.getDirectory();
					nameSave = new File(helperFrame.getFile());

				} catch (Exception e1) {
					message("You cancelled the saving.");
					break;
				}

				if ((pathSave == null) || (nameSave == null)) {

					/*
					 * Unter gewissen Umständen werden die Datei- und Pfadnamen,
					 * die hier ausgewählt wurden, von Java nicht erkannt.
					 * Beispielsweise dann, wenn man die Liste der zuletzt
					 * benutzen Speicherorte zum Auswählen des aktuellen
					 * Speicherorts verwendet. Das ist ein Problem, für das wir
					 * als Zweitsemestrige nicht zuständig sind.
					 */

					message("You must not use the \"Recently used\" method"
							+ " of selecting file names.\nYou will now get another"
							+ " chance.");
				}

			} while ((pathSave == null) || (nameSave == null));

			if ((pathSave == null) || (nameSave == null))
				return;
			save(nameSave, pathSave);

			break;

		case "Load":

			if (getMyOcean().isOceanRunning()) {
				getMyOceanGUI().getButtonPanel().getStartStopButton().doClick();
			}

			File nameLoad = null;
			String pathLoad = null;

			do {
				FileDialog helperFrame = new FileDialog(
						(Frame) getMyOceanGUI(), "Load", FileDialog.LOAD);
				helperFrame.setVisible(true);
				try {
					pathLoad = helperFrame.getDirectory();
					nameLoad = new File(helperFrame.getFile());
				} catch (Exception e1) {
					message("You cancelled the loading.");
					break;
				}
				if ((pathLoad == null) || (nameLoad == null)) {
					/* siehe oben */

					message("You must not use the \"Recently used\" method"
							+ " of selecting file names.\nYou will now get another"
							+ " chance.");
				}

			} while ((pathLoad == null) || (nameLoad == null));
			if ((pathLoad == null) || (nameLoad == null))
				return;

			load(nameLoad, pathLoad);
			updateGUI();
			break;

		default:
			System.err.println("Hier stimmt was nicht");
		}
	}

	/**
	 *
	 * @param type
	 *            a String describing the type of OceanObject to create
	 * @param name
	 *            a String containing the name for the object
	 * @param maximumAge
	 *            an integer specifying the maximum age
	 * @param position
	 * @throws Exception
	 *
	 *             Create an new OceanObject, by type.
	 *
	 */
	private void createObject(String type, String name, int maximumAge,
			int[] position) throws Exception {

		switch (type) {

		case "Fish":
			new Fish(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;

		case "Shark":
			new Shark(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;

		case "Stone":
			new Stone(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;

		case "Bubble":
			new Bubble(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;

		case "Plant":
			new Plant(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;

		case "Coke Can":
			new CokeCan(makeOOID(), name, position, getMyOcean(), maximumAge);
			break;
		}
	}

	/**
	 * Gets the maximum age.
	 *
	 * @return the maximum age
	 */
	private int getMaximumAge() {

		int maximumAge = -1;
		String maximumAgeString;

		try {
			maximumAgeString = getMyOceanGUI().getButtonPanel()
					.getMaximumAgeField().getText();
			if ((maximumAgeString == null) || (maximumAgeString.length() == 0))
				throw new EmptyStringException();

			if (!maximumAgeString.equalsIgnoreCase(maximumAgeString.trim()))
				throw new ExcessWhitespaceException();

			maximumAge = Integer.parseInt(maximumAgeString);

			if (maximumAge < 0)
				throw new MaximumAgeNegativeException();

		} catch (EmptyStringException e1) {
			message("You must supply a maximum age");
			return -1;
		}
		catch (NumberFormatException e2) {
			message("Maximum age must be an integer");
			return -1;
		}
		catch (ExcessWhitespaceException e) {
			message("There is excess white space in the maximum age input field");
		}
		catch (MaximumAgeNegativeException e) {
			message("The maximum age must not be negative");
			return -1;
		}
		catch (Exception e1) {
			message("Something's wrong with the maximum age");
			return -1;
		}
		return maximumAge;
	}

	/**
	 * @return The String from the name field.
	 */
	private String getName() {

		String name = null;

		try {
			name = getMyOceanGUI().getButtonPanel().getNameField().getText();

			if ((name == null) || (name.length() == 0))
				throw new EmptyStringException();

		} catch (EmptyStringException e1) {
			message("You must supply a name");
			return null;
		}
		catch (Exception e1) {
			message("Something's wrong with the name");
			return null;
		}

		return name;

	}

	/**
	 * Gets the type of object.
	 *
	 * @return the type of object
	 */
	private String getTypeOfObject() {

		return getMyOceanGUI().getButtonPanel().getOceanObjectTypesString();

	}

	/**
	 * Gets the x position.
	 *
	 * @return the x position
	 */
	private int getXPosition() {

		int xPosition = -1;

		String xPositionString;

		try {

			xPositionString = getMyOceanGUI().getButtonPanel().getXValueField()
					.getText();

			if ((xPositionString == null) || (xPositionString.length() == 0))
				throw new EmptyStringException();

			if (!xPositionString.equalsIgnoreCase(xPositionString.trim()))
				throw new ExcessWhitespaceException();

			xPosition = Integer.parseInt(xPositionString);

		} catch (EmptyStringException e1) {
			message("You must supply an x coordinate for the position");
			return -1;
		}
		catch (NumberFormatException e2) {
			message("The position's x coordinate must be an integer");
			return -1;
		}
		catch (ExcessWhitespaceException e) {
			message("There is excess white space in the x coordinate input field");
		}
		catch (Exception e1) {
			message("Something's wrong with the x coordinate");
			return -1;
		}
		return xPosition;
	}

	/**
	 * Gets the y position.
	 *
	 * @return the y position
	 */
	private int getYPosition() {

		int yPosition = -1;
		String yPositionString;
		try {

			yPositionString = getMyOceanGUI().getButtonPanel().getYValueField()
					.getText();

			if ((yPositionString == null) || (yPositionString.length() == 0))
				throw new EmptyStringException();

			if (!yPositionString.equalsIgnoreCase(yPositionString.trim()))
				throw new ExcessWhitespaceException();

			yPosition = Integer.parseInt(yPositionString);

		} catch (EmptyStringException e1) {
			message("You must supply a y coordinate for the position");
			return -1;
		}
		catch (NumberFormatException e2) {
			message("The position's y coordinate must be an integer");
			return -1;
		}
		catch (ExcessWhitespaceException e) {
			message("There is excess white space in the y coordinate input field");
		}
		catch (Exception e1) {
			message("Something's wrong with the y coordinate");
			return -1;
		}
		return yPosition;
	}

	/**
	 * This method loads an ocean from a file.
	 *
	 * @param nameLoad
	 *            a String containig the file name.
	 * @param pathLoad
	 *            a String containig the path to the file.
	 */
	private void load(File nameLoad, String pathLoad) {
		try {

			Boolean loaded = getMyOcean().load(pathLoad, nameLoad);

			if (loaded) {
				message("Successfully loaded ocean");
			} else {
				message("Failed to load ocean");
			}
		} catch (Exception e1) {
			message("Failed to load ocean");
		}
	}

	/**
	 * Prints out the state of each bubble generator thread,
	 * for debugging purposes.
	 */
	private void printBubbleThreadStates() {
		synchronized (getMyOcean().getOceanObjects()) {
			for (OceanObject obj : getMyOcean().getOceanObjects()) {
				if (obj.getType() == COKE_CAN) {
					CokeCan c = (CokeCan) obj;
					Thread t = c.getCcbgt();
					if (t != null) {
						System.out.println("stepButton: ccbgt of "
								+ obj.getOceanObjectName() + " is "
								+ t.getState());
					}
				}
			}
		}
	}

	/**
	 * This method saves an ocean to a file.
	 *
	 * @param nameSave
	 *            a String containig the file name.
	 * @param pathSave
	 *            a String containig the path to the file.
	 */
	private void save(File nameSave, String pathSave) {
		try {

			Boolean saved = getMyOcean().save(pathSave, nameSave);
			if (saved) {
				message("Successfully saved ocean");
			} else {
				message("Failed to save ocean");
			}
		} catch (Exception e1) {
			message("Failed to save ocean");
		}
	}

	/**
	 * Print out a little ocean overview.
	 */
	private void showOverview() {
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(FISH)
				+ " fish in ocean");
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(SHARK)
				+ " sharks in ocean");
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(STONE)
				+ " stones in ocean");
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(BUBBLE)
				+ " bubbles in ocean");
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(PLANT)
				+ " plants in ocean");
		System.out.println(getMyOcean().getNumberOfOceanObjectsByType(COKE_CAN)
				+ " cans in ocean");
	}
}