package controller;

import model.OceanInterface;
import view.OceanGUIInterface;

/**
 * The Class Controller.
 */
public class Controller {

	/** The ocean is a part of the model. */
	private OceanInterface myOcean;

	/** The oceanGUI is a part of the GUI */
	private OceanGUIInterface myOceanGUI;

	public Controller(OceanInterface model, OceanGUIInterface view) {

		setMyOceanGUI(view);
		setMyOcean(model);
		addButtonListeners();
		addMouseControllers();
	}

	// _____________________________________________________________________________

	/**
	 * Gets the my ocean attribute.
	 * 
	 * @return myOcean the ocean we work with
	 */
	private OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * Gets the my oceanGUI attribute.
	 * 
	 * @return the myOceanGUI the GUI we work with
	 */
	private OceanGUIInterface getMyOceanGUI() {
		return myOceanGUI;
	}

	/**
	 * Sets the myOcean attribute.
	 * 
	 * @param myOcean the ocean we work with
	 */
	private void setMyOcean(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the myOceanGUI attribute.
	 * 
	 * @param view the GUI we work with
	 */
	private void setMyOceanGUI(OceanGUIInterface view) {
		this.myOceanGUI = view;
	}
	/**
	 * Adds the button listeners to OceanGUI.
	 */
	private void addButtonListeners() {
		ButtonListener bc = new ButtonListener(getMyOcean(), getMyOceanGUI());
		getMyOceanGUI().getButtonPanel().addListeners(bc);
	}

	/**
	 * Adds the mouse controller to OceanGUI.
	 */
	private void addMouseControllers() {

		( getMyOceanGUI()).addMouseListeners(new MouseListener(
				getMyOcean(), getMyOceanGUI()), new MouseListener(
						getMyOcean(), getMyOceanGUI()));
	}
}