package controller;

import model.OceanInterface;
import view.OceanGUIInterface;

/**
 * 
 * This class contains all those methods that both the ButtonListener and
 * MouseListener can use.
 * 
 */
public class Listener {

	/**
	 * The Class EmptyStringException.
	 * Instances of this class are thrown whenever the input in the
	 * name, x, y, maximum age fields is empty.
	 */
	protected class EmptyStringException extends Exception {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = 7312146429306850592L;

	}

	/**
	 * The view interface
	 */
	private OceanGUIInterface myOceanGUI;
	/**
	 * The model interface
	 */
	private OceanInterface myOcean;

	// ______________________________________________________________________________________________________

	/**
	 * 
	 * @param model
	 *            the ocean we work with
	 * @param view
	 *            the GUI we work with
	 */
	public Listener(OceanInterface model, OceanGUIInterface view) {

		setMyModel(model);
		setMyView(view);
	}

	/**
	 * @return myOcean the OceanInterface that describes the ocean we work with
	 */
	protected OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * @return myOceanGUI the OceanGUIInterface that describes the GUI we work with
	 */
	protected OceanGUIInterface getMyOceanGUI() {
		return myOceanGUI;
	}

	/**
	 * 
	 * @param errorMsg a String that is displayed in the message
	 */
	protected void message(String errorMsg) {
		getMyOceanGUI().showErrorMessage(errorMsg);
	}

	// ______________________________________________________________________________________________________

	/**
	 * Repaints the ocean.
	 */
	protected void repaintOcean() {
		getMyOceanGUI().getWaterPanel().repaint();
	}

	/**
	 * Sets the myOcean attribute.
	 * 
	 * @param myOcean the ocean to set
	 */
	protected void setMyModel(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the myOceanGUI attribute.
	 * 
	 * @param myOceanGUI the OceanGUI to set
	 * 
	 */
	protected void setMyView(OceanGUIInterface myOceanGUI) {
		this.myOceanGUI = myOceanGUI;
	}

	/**
	 * This method let sleep up an unspecific Thread.
	 * 
	 * @param ms the time to sleep, in milliseconds
	 * 
	 */
	protected void sleep(int ms) {
		try {
			Thread.sleep(ms);
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Updates the whole GUI.
	 */
	protected void updateGUI() {
		repaintOcean();
		updateStatusPanel();
	}

	/**
	 * Updates the status panel.
	 */
	protected void updateStatusPanel() {
		getMyOceanGUI().getStatusPanel().update();
	}
}
