package model;

import static java.lang.Math.abs;
import static model.Ocean.BUBBLE;
import static model.Ocean.PLANT;

/**
 * The Class Bubble.
 */
public class Bubble extends OceanObject {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -7019931214166722276L;

	/**
	 * Instantiates a new bubble.
	 *
	 * @param ID the id
	 * @param name the name
	 * @param position the position
	 * @param ocean the ocean
	 * @param maximumAge the maximum age
	 * @throws Exception the exception
	 */
	public Bubble(int ID, String name, int[] position, OceanInterface ocean,
			int maximumAge) throws Exception {

		super(ID, name, position, ocean, maximumAge);

		setType(BUBBLE);
		setxDirConst(3);
		setyDirConst(-5);
		setxDirVar(-8);
		setyDirVar(2);
	}

	// __________________________________________________________


	/**
	 * This method describes the way an object reacts after a collision.
	 * 
	 * @param obj2 the second one of the collision
	 */
	@Override
	public void reaktion(OceanObject obj2) {

		switch (obj2.getType()) {

		case BUBBLE:
			getMyOcean().removeOceanObject(this);
			break;

		case PLANT:
			setDirection(obj2.getDirection());
			break;

		default:
			break;
		}
	}

	/**
	 * Describes the way an objects reacts with the oceans bounds.
	 * 
	 * @param collisionDetected is a collision detected
	 * 
	 */
	@Override
	protected void updateDirectionWhenCollidingWithOceanBounds(
			int collisionDetected) {

		switch (collisionDetected) {

		case KOLLISION_RECHTS:
			setxDirConst(-abs(getxDirConst()));
			setxDirVar(abs(getxDirVar()));
			setDirectionX(-abs(getDirectionX()));
			break;

		case KOLLISION_OBEN:
			getMyOcean().removeOceanObject(this);
			break;

		case KOLLISION_UNTEN:
			setyDirConst(-abs(getyDirConst()));
			setyDirVar(abs(getyDirVar()));
			break;

		case KOLLISION_LINKS:
			setxDirConst(abs(getxDirConst()));
			setxDirVar(-abs(getxDirVar()));
			setDirectionX(abs(getDirectionX()));
			break;

		case KEINE_KOLLISION:
			break;

		default:
			break;
		}
	}
}
