package model;

import static java.lang.Math.abs;
import static model.Ocean.COKE_CAN;
import static model.Ocean.FISH;
import static model.Ocean.PLANT;
import static model.Ocean.SHARK;
import static model.Ocean.STONE;

/**
 * The Class CokeCan.
 */
public class CokeCan extends OceanObject {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 7401838673828408123L;

	/** The ccbgt. */
	private CokeCanBubbleGeneratorThread ccbgt = null;

	/** The has reached bottom. */
	private Boolean hasReachedBottom = false;

	/**
	 * Instantiates a new coke can.
	 *
	 * @param ID the id
	 * @param name the name
	 * @param position the position
	 * @param ocean the ocean
	 * @param maximumAge the maximum age
	 * @throws Exception the exception
	 */
	public CokeCan(int ID, String name, int[] position, OceanInterface ocean,
			int maximumAge) throws Exception {

		super(ID, name, position, ocean, maximumAge);
		setType(COKE_CAN);
		setUpright(true);
		setxDirConst(0);
		setyDirConst(1);
		setxDirVar(1);
		setyDirVar(5);
	}

	// ________________________________________________________________

	/**
	 * Gets the CokeCanBubbleGeneratorThread.
	 *
	 * @return the ccbgt
	 */
	public CokeCanBubbleGeneratorThread getCcbgt() {
		return ccbgt;
	}

	/**
	 * Sets the ccbgt.
	 *
	 * @param ccbgt the new ccbgt
	 */
	public void setCcbgt(CokeCanBubbleGeneratorThread ccbgt) {
		this.ccbgt = ccbgt;
	}

	/**
	 * Checks for reached bottom.
	 *
	 * @return the hasReachedBottom
	 */
	public Boolean hasReachedBottom() {
		return hasReachedBottom;
	}

	/**
	 * Sets the checks for reached bottom.
	 *
	 * @param hasReachedBottom            the hasReachedBottom to set
	 */
	public void setHasReachedBottom(Boolean hasReachedBottom) {
		this.hasReachedBottom = hasReachedBottom;
	}

	/**
	 * This method describes the way an object reacts after a collision.
	 * 
	 * @param obj2 the second one of the collision
	 */
	@Override
	public synchronized void reaktion(OceanObject obj2) {

		switch (obj2.getType()) {

		case FISH:
			youBounce(obj2);
			if (getUpright()) {
				if (getPositionX() > obj2.getPositionX()) {
					toppleRight();
				} else {
					toppleLeft();
				}
			}
			break;

		case SHARK:
			youBounce(obj2);
			if (getUpright()) {
				if (getPositionX() > obj2.getPositionX()) {
					toppleRight();
				} else {
					toppleLeft();
				}
			}
			break;

		case STONE:
			// Dose entfernen, wie bei Steinen
			crushLowerObject(obj2);
			break;

		case PLANT:
			crushLowerObject(obj2);
			break;

		case COKE_CAN:
			crushLowerObject(obj2);
			break;

		default:
			break;
		}
	}

	/**
	 * Describes the way an objects reacts with the oceans bounds.
	 * 
	 * @param collisionDetected is a collision detected
	 * 
	 */
	@Override
	protected void updateDirectionWhenCollidingWithOceanBounds(
			int collisionDetected) {

		switch (collisionDetected) {

		case KOLLISION_RECHTS:
			setxDirConst(-abs(getxDirConst()));
			setDirectionX(-abs(getDirectionX()));
			break;

		case KOLLISION_OBEN:
			break;
		case KOLLISION_UNTEN:

			setDirection(new int[] { 0, 0 });
			setxDirConst(0);
			setyDirConst(0);
			setxDirVar(0);
			setyDirVar(0);

			if (!hasReachedBottom()) {

				ccbgt = new CokeCanBubbleGeneratorThread(getMyOcean(), this,
						getID());
				ccbgt.start();
			}
			setHasReachedBottom(true);

			break;

		case KOLLISION_LINKS:
			setxDirConst(abs(getxDirConst()));
			setDirectionX(abs(getDirectionX()));
			break;
		case KEINE_KOLLISION:
			break;

		default:
			break;

		}
	}
}