package model;

import java.io.Serializable;
import java.util.Random;

/**
 * The Class CokeCanBubbleGeneratorThread.
 */
public class CokeCanBubbleGeneratorThread extends Thread implements
Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6231430945266122750L;

	/** The bubble position. */
	private final int[] bubblePosition = new int[] { -1, -1 };

	/** The lockObject. */
	private Integer lock;

	/** The CokeCan. */
	private CokeCan myCokeCan;

	/** The ocean. */
	private OceanInterface myOcean;

	/** The name. */
	private String name = null;

	/** The nozzle position. */
	private int[] nozzlePosition = new int[] { -1, -1 };

	/** A random value generator. */
	private Random rand = new Random();

	// _____________________________________________________________________________________________________

	/**
	 * Instantiates a new coke can bubble generator thread.
	 *
	 * @param oceanInterface the ocean interface
	 * @param c the CokeCan
	 * @param lock the lockObject
	 */
	public CokeCanBubbleGeneratorThread(OceanInterface oceanInterface,
			CokeCan c, Integer lock) {

		setMyOcean(oceanInterface);
		setMyCokeCan(c);
		setLock(lock);

		setCokeCanName(getMyCokeCan().getOceanObjectName() + " ("
				+ getMyCokeCan().getID() + "): ");
		if (name == null) {
			System.exit(7);
		}
	}

	// _____________________________________________________________________________________________________
	/**
	 * Sets the coke can name.
	 *
	 * @param name the new coke can name
	 */
	public void setCokeCanName(String name) {
		this.name = name;
	}

	/**
	 * Sets the lock.
	 *
	 * @param lock the new lock
	 */
	public void setLock(Integer lock) {
		this.lock = lock;
	}

	/**
	 * Sets the coke can.
	 *
	 * @param myCokeCan the CokeCan to set
	 */
	public void setMyCokeCan(CokeCan myCokeCan) {
		this.myCokeCan = myCokeCan;
	}

	/**
	 * Sets the ocean.
	 *
	 * @param myOcean the ocean to set
	 */
	public void setMyOcean(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the nozzle position.
	 *
	 * @param nozzlePosition the nozzlePosition to set
	 */
	public void setNozzlePosition(int[] nozzlePosition) {
		this.nozzlePosition = nozzlePosition;
	}

	/**
	 * Sets the random number generator.
	 *
	 * @param rand the random number generator to set
	 */
	public void setRand(Random rand) {
		this.rand = rand;
	}

	/**
	 * Gets the CokeCan name.
	 *
	 * @return the cokecan name
	 */
	public String getCokeCanName() {
		return name;
	}

	/**
	 * Gets the lockObject.
	 *
	 * @return the lockObject
	 */
	public Integer getLock() {
		return lock;
	}

	/**
	 * Gets the coke can.
	 *
	 * @return the myCokeCan
	 */
	public CokeCan getMyCokeCan() {
		return myCokeCan;
	}

	/**
	 * Gets the ocean.
	 *
	 * @return the ocean
	 */
	public OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * Gets the nozzle position.
	 *
	 * @return the nozzlePosition
	 */
	public int[] getNozzlePosition() {
		return nozzlePosition;
	}

	/**
	 * Gets the nozzle position's x value.
	 *
	 * @return the nozzle position's x value
	 */
	private int getNozzlePositionX() {
		return getNozzlePosition()[0];
	}

	/**
	 * Gets the nozzle position's y value.
	 *
	 * @return the nozzle position's y value
	 */
	private int getNozzlePositionY() {
		return getNozzlePosition()[1];
	}

	/**
	 * Gets the random number generator.
	 *
	 * @return the random number generator
	 */
	public Random getRand() {
		return rand;
	}

	/**
	 * Calculates the nozzle position.
	 */
	private void calcNozzlePosition() {

		if (getMyCokeCan().getUpright()) {
			setNozzlePosition(new int[] { 28, -10 });

		} else if (getMyCokeCan().getToppledLeft()) {
			setNozzlePosition(new int[] { -6, 0 });

		} else {
			setNozzlePosition(new int[] { 82, 20 });
		}
	}

	/**
	 * Makes a new bubble with correct position.
	 */
	private void makeNewBubble() {
		calcNozzlePosition();

		bubblePosition[0] = getMyCokeCan().getPositionX()
				+ getNozzlePositionX() + getRand().nextInt(1);

		bubblePosition[1] = (getMyCokeCan().getPosition()[1] + getNozzlePositionY())
				- getRand().nextInt(1);

		try {
			new Bubble(Ocean.makeOOID(), "Testbubble", bubblePosition,
					getMyOcean(), 5000);
		} catch (Exception e) {
			System.out.println("neue Bubble konnte nicht erzeugt werden");
		}
	}

	/**
	 * Generates a new Bubble and lets the thread sleep for a while.
	 */
	@Override
	public void run() {

		while (getMyOcean().contains(getMyCokeCan())) {

			synchronized (getLock()) {

				if (getMyOcean().isOceanRunning()) {

					makeNewBubble();
					sleep();

				} else {
					try {

						lock.wait();

					} catch (InterruptedException e) {
						System.err.println(name + "schlaflos: "
								+ this.getState());
					}
				}
			}
		}
	}

	/**
	 * Let an unspecified thread sleep.
	 * 
	 */
	private void sleep() {
		try {
			Thread.sleep(400);

			// Bitte nicht kleiner als 400 setzen, die Blasen müssen die
			// möglichkeit haben den Spawn-Punkt zu verlassen, bevor eine
			// neue spawnt.
			// Hier kann man unnötige Fehler werfen :)

		} catch (Exception e) {
		}
	}
}
