package model;

import static java.lang.Math.abs;
import static java.lang.Math.min;
import static java.lang.Math.random;
import static model.Ocean.COKE_CAN;
import static model.Ocean.FISH;
import static model.Ocean.PLANT;
import static model.Ocean.SHARK;
import static model.Ocean.STONE;

/**
 * The Class Fish.
 */
public class Fish extends OceanObject {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 8127821248332651013L;

	/**
	 * Instantiates a new fish.
	 *
	 * @param ID the id
	 * @param name the name
	 * @param position the position
	 * @param ocean the ocean
	 * @param maximumAge the maximum age
	 * @throws Exception the exception
	 */
	public Fish(int ID, String name, int[] position, OceanInterface ocean,
			int maximumAge) throws Exception {

		super(ID, name, position, ocean, maximumAge);

		setType(FISH);
		setxDirConst(8);
		setyDirConst(-3);
		setxDirVar(2);
		setyDirVar(2);
	}

	// ________________________________________________________________

	/**
	 * This method describes the way an object reacts after a collision.
	 * 
	 * @param obj2 the second one of the collision
	 */
	@Override
	public synchronized void reaktion(OceanObject obj2) {

		switch (obj2.getType()) {
		case FISH:

			int room = min(getMyOcean().getWidth(), getMyOcean().getDepth());
			int fish = getMyOcean().getNumberOfOceanObjectsByType(FISH);

			int maxfish = (int) (((1.0 / 15) * room) - (int) (random() * 20));

			// die zahl maxfish nur behutsam ändern, damit die fische nicht
			// aussterben

			if (fish < maxfish) {

				if ((getAge() >= (getMaximumAge() / 2))
						&& (obj2.getAge() >= (obj2.getMaximumAge() / 2))) {

					int baseAge = room / 4;

					int maxAge = min(2,
							(int) ((baseAge + ((1.0 / 6) * room)) - (3 * fish)));

					spawnRandomFish(maxAge);
					obj2.setAge((getMaximumAge() * 96) / 100);
				}
			}

			break;

		case STONE:
			meBounce(obj2);
			break;

		case PLANT:
			obj2.setAge((int) (getMaximumAge() * 0.9));
			setMaximumAge((int) (getMaximumAge() * 1.5));
			break;

		case SHARK:
			int r = abs(getRand().nextInt()) % 5;
			if (r == 0) {
				getMyOcean().removeOceanObject(this);
				obj2.decHunger();
				obj2.setMaximumAge((int) (getMaximumAge() * 1.1));
			}
			meBounce(obj2);
			break;

		case COKE_CAN:
			meBounce(obj2);
			topple(obj2);
			break;

		default:
			break;
		}
	}

	/**
	 * Spawn random fish.
	 *
	 * @param maximumAge the maximum age
	 */
	private void spawnRandomFish(int maximumAge) {

		int[] spawnPosition = new int[2];

		spawnPosition[0] = getRand().nextInt(
				(int) (((getMyOcean().getWidth()) - getDimensionsOfIcon()
						.getWidth()) + 1));

		spawnPosition[1] = getRand().nextInt(
				(int) (((getMyOcean().getDepth()) - getDimensionsOfIcon()
						.getHeight()) + 1));

		try {

			new Fish(Ocean.makeOOID(), "Fishy", spawnPosition, getMyOcean(),
					getMaximumAge());
		} catch (Exception e) {
		}
	}

	/**
	 * Describes the way an object reacts with the oceans bounds.
	 * 
	 * @param collisionDetected is a collision detected
	 * 
	 */
	@Override
	protected void updateDirectionWhenCollidingWithOceanBounds(
			int collisionDetected) {

		switch (collisionDetected) {

		case KOLLISION_RECHTS:
			setxDirConst(-abs(getxDirConst()));
			setxDirVar(-getxDirVar());
			setDirectionX(-getDirectionX());
			setDirectionY(-getDirectionY());
			setPositionX(getMyOcean().getWidth() - getDimensionsOfIcon().width);
			break;

		case KOLLISION_OBEN:
			setDirection(new int[] { getDirectionX(), -getDirectionY() });
			setyDirConst(-getyDirConst());
			setyDirVar(-getyDirVar());

			setPositionY(0);
			break;

		case KOLLISION_UNTEN:
			setDirection(new int[] { getDirectionX(), -getDirectionY() });
			setyDirConst(-getyDirConst());
			setyDirVar(-getyDirVar());
			break;

		case KOLLISION_LINKS:
			setxDirConst(-getxDirConst());
			setxDirVar(-getxDirVar());
			setDirectionX(-getDirectionX());
			setDirectionY(-getDirectionY());
			setPositionX(0);

			break;

		case KEINE_KOLLISION:
			break;

		default:
			break;

		}
	}
}