package model;

import java.io.Serializable;

/**
 * The Class MovementThread.
 */
public class MovementThread extends Thread implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The lock object. */
	private Integer lock;

	/** The ocean. */
	private Ocean myOcean;

	// _______________________________________________________________________________________________________________________
	/**
	 * Instantiates a new movement thread.
	 *
	 * @param o the ocean
	 * @param lock the lock object
	 */
	public MovementThread(Ocean o, Integer lock) {

		setMyOcean(o);
		setLock(lock);
	}

	/**
	 * Gets the lock object.
	 *
	 * @return the lock
	 */
	public Integer getLock() {
		return lock;
	}

	/**
	 * Gets the ocean.
	 *
	 * @return the ocean
	 */
	private Ocean getMyOcean() {
		return myOcean;
	}

	/**
	 * Sets the lock.
	 *
	 * @param lock the new lock
	 */
	public void setLock(Integer lock) {
		this.lock = lock;
	}

	/**
	 * Sets the my ocean.
	 *
	 * @param o the new my ocean
	 */
	private void setMyOcean(Ocean o) {
		this.myOcean = o;
	}

	/**
	 * Moves the ocean.
	 */
	private void moveOcean() {
		try {
			synchronized (getMyOcean()) {

				getMyOcean().move();
			}

		} catch (Exception e1) {
			System.out.println("failed to move the ocean");
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {

		while (true) {
			synchronized (getLock()) {

				while (getMyOcean().isOceanRunning()) {

					moveOcean();
					sleep();
				}
				try {
					getLock().wait();

				} catch (InterruptedException e) {
				}
			}
		}
	}

	/**
	 * Sleep.
	 */
	private void sleep() {
		try {
			Thread.sleep(getMyOcean().getAnimationDelay());
		} catch (InterruptedException e) {
			System.err.println("MovementThread: I can't get no sleep...");
		}
	}
}
