package model;

import static java.lang.Math.abs;
import static java.lang.Math.random;
import static model.Ocean.COKE_CAN;
import static model.Ocean.FISH;
import static model.Ocean.PLANT;
import static model.Ocean.SHARK;
import static model.Ocean.STONE;

import java.awt.Dimension;
import java.io.Serializable;
import java.util.Random;

import javax.swing.ImageIcon;

/**
 * The Class OceanObject.
 */
public abstract class OceanObject implements Serializable {

	/** The Constant KEINE_KOLLISION. */
	protected static final int KEINE_KOLLISION = 0;

	/** The Constant KOLLISION_LINKS. */
	protected static final int KOLLISION_LINKS = 3;

	/** The Constant KOLLISION_OBEN. */
	protected static final int KOLLISION_OBEN = 2;
	// OceanBorderProblems
	/** The Constant KOLLISION_RECHTS. */
	protected static final int KOLLISION_RECHTS = 1;

	/** The Constant KOLLISION_UNTEN. */
	protected static final int KOLLISION_UNTEN = 4;

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The age of an oceanobject. */
	private int age = 0;

	/** The direction is like a movement vector. */
	protected int[] direction = new int[] { 0, 0 };

	/** The hunger. */
	private int hunger;

	/** The icon of an ocean object. */
	private ImageIcon icon;

	/** The id is the final identifier. */
	private int ID;

	/** The maximum age. */
	private int maximumAge;

	/** The model interface. */
	private OceanInterface myOcean;

	/** The object's name. */
	private String oceanObjectName;

	/** The current position. */
	private int[] position;

	/** A random value for a more natural movement. */
	protected Random rand;

	/** This attribute is 'true', when the Object is tagged. */
	private Boolean tagged = false;

	/** This attribute is 'true' if the object lies on the left side.  */
	protected Boolean toppledLeft = false;

	/** This attribute is 'true' if the object lies on the right side.  */
	protected Boolean toppledRight = false;

	/** The type value describes what type of ocean object the object is (like: Fish, Stone, Bubble...) */
	private int type;

	/** This attribute is 'true', when the Object is upright. */
	protected Boolean upright = true;

	/** The x-dir-const value. */
	private int xDirConst;

	/** The x-dir-var value. */
	private int xDirVar;

	/** The y-dir-const value. */
	private int yDirConst;

	/** The y-dir-var value. */
	private int yDirVar;

	// _____________________________________________________________________________

	/**
	 * Instantiates a new OceanObject.
	 *
	 * @param ID the id
	 * @param name the name
	 * @param position the position
	 * @param ocean the ocean
	 * @param maximumAge the maximum age
	 * @throws Exception the exception
	 */
	public OceanObject(int ID, String name, int[] position,
			OceanInterface ocean, int maximumAge) throws Exception {

		setID(ID);
		setOceanObjectName(name);
		setPosition(position);
		setMaximumAge(maximumAge);
		setAge(0);
		setMyOcean(ocean);
		setDirection(new int[] { getxDirConst(), getyDirConst() });
		setRand(new Random());

		setIcon(getClass().getCanonicalName().toLowerCase()
				.replace("model.", "")
				+ ".png");

		getMyOcean().addOceanObject(this);
	}

	// _____________________________________________________________________________



	/**
	 * Decrease the attribute hunger by one.
	 */
	public void decHunger() {
		hunger--;
	}

	/**
	 * Gets the oceanObject age.
	 *
	 * @return age the ocean object's age
	 */
	public int getAge() {
		return age;
	}

	/**
	 * Gets the dimensions of the object's icon.
	 *
	 * @return a Dimension containing the width and height of the object's icon.
	 */
	public Dimension getDimensionsOfIcon() {

		Dimension iconDimension = new Dimension(0, 0);
		iconDimension.height = icon.getIconHeight();
		iconDimension.width = icon.getIconWidth();
		return iconDimension;
	}

	/**
	 * Gets the direction.
	 *
	 * @return the direction
	 */
	public int[] getDirection() {
		return direction;
	}

	/**
	 * Gets the direction as a string.
	 *
	 * @return the direction string
	 */
	public String getDirectionString() {
		return "(" + getDirectionX() + ", " + getDirectionY() + ")";
	}

	/**
	 * Gets the direction's x-value.
	 *
	 * @return the direction's x-value
	 */
	public int getDirectionX() {
		return getDirection()[0];
	}

	/**
	 * Gets the direction's y-value.
	 *
	 * @return the direction's y-value
	 */
	public int getDirectionY() {
		return getDirection()[1];
	}

	/**
	 * Gets the hunger.
	 *
	 * @return the hunger
	 */
	public int getHunger() {
		return hunger;
	}

	/**
	 * Gets the icon.
	 *
	 * @return the icon
	 */
	public ImageIcon getIcon() {
		return icon;
	}

	/**
	 * Gets the oceanObject's id.
	 *
	 * @return the id
	 */
	public int getID() {
		return ID;
	}

	/**
	 * Gets the oceanObjects maximum age.
	 *
	 * @return the maximumAge
	 */
	public int getMaximumAge() {
		return maximumAge;
	}

	/**
	 * Gets the model interface.
	 *
	 * @return the ocean interface
	 */
	public OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * Gets the oceanObject's name.
	 *
	 * @return the oceanObject's name
	 */
	public String getOceanObjectName() {
		return oceanObjectName;
	}

	/**
	 * Gets the position in an int[2]-Array.
	 *
	 * @return the position
	 */
	public int[] getPosition() {
		return position;
	}

	/**
	 * Gets the position as a string.
	 *
	 * @return the position string
	 */
	public String getPositionString() {
		return "(" + getPositionX() + ", " + getPositionY() + ")";
	}

	/**
	 * Gets the position's x value.
	 *
	 * @return the position's x value
	 */
	public int getPositionX() {
		return getPosition()[0];
	}

	/**
	 * Gets the position's y value.
	 *
	 * @return the position's y value
	 */
	public int getPositionY() {
		return getPosition()[1];
	}

	/**
	 * Gets the random number generator attribute.
	 *
	 * @return the random number generator
	 */
	public Random getRand() {
		return rand;
	}

	/**
	 * Gets the tagged attribute.
	 *
	 * @return whether or not the object is tagged
	 */
	public Boolean getTagged() {
		return tagged;
	}

	/**
	 * Gets the toppled-left attribute.
	 *
	 * @return whether or not the object is toppled left
	 */
	public Boolean getToppledLeft() {
		return toppledLeft;
	}

	/**
	 * Gets the toppled-right attribute.
	 *
	 * @return whether or not the object is toppled right
	 */
	public Boolean getToppledRight() {
		return toppledRight;
	}

	/**
	 * Gets the type.
	 *
	 * @return the type of OceanObject
	 */
	public int getType() {
		return type;
	}

	/**
	 * Gets the upright attribute.
	 *
	 * @return whether or not the object is upright
	 */
	public Boolean getUpright() {
		return upright;
	}

	/**
	 * Gets the x-dir-const value.
	 *
	 * @return the xDirConst value
	 */
	public int getxDirConst() {
		return xDirConst;
	}

	/**
	 * Gets the x-dir-var value.
	 *
	 * @return the xDirVar value
	 */
	public int getxDirVar() {
		return xDirVar;
	}

	/**
	 * Gets the y-dir-const value.
	 *
	 * @return the yDirConst value
	 */
	public int getyDirConst() {
		return yDirConst;
	}

	/**
	 * Gets the y-dir-var value.
	 *
	 * @return the yDirVar value
	 */
	public int getyDirVar() {
		return yDirVar;
	}

	/**
	 * Increases the hunger attribute.
	 */
	public void incHunger() {
		hunger++;
	}
	/**
	 * Finds out whether or not this ocean object collides with the ocean's bounds.
	 *
	 * @return an integer specifying whether or not this ocean object collides with the ocean's bounds
	 */
	public int collidesWithOceanBounds() {

		int objPosX = getPositionX();
		int objPosY = getPositionY();
		int objWidth = getIcon().getIconWidth();
		int objhigth = getIcon().getIconHeight();
		int objDirX = getDirectionX();
		int objDirY = getDirectionY();

		if ((objPosX + objWidth + objDirX) > getMyOcean().getWidth())
			return KOLLISION_RECHTS;

		if ((objPosX + objDirX) < 0)
			return KOLLISION_LINKS;

		if ((objPosY + objhigth + objDirY) > getMyOcean().getDepth())
			return KOLLISION_UNTEN;

		if ((objPosY + objDirY) < 0)
			return KOLLISION_OBEN;

		return KEINE_KOLLISION;
	}

	/**
	 * Crush the lower object.
	 * The lower object will be removed form the ocean, also from the oceans OceanObjectList.
	 *
	 * @param obj2 the other object
	 */
	public void crushLowerObject(OceanObject obj2) {

		if (getPositionY() > obj2.getPositionY()) {

			if (getDirectionY() == 0) {
				getMyOcean().removeOceanObject(this);
			}

		} else {

			if (obj2.getDirectionY() == 0) {
				getMyOcean().removeOceanObject(obj2);
			}
		}
	}

	/**
	 * Me bounce.
	 * This method lets this object bounce from obj2.
	 *
	 * @param obj2 is the second object in the collision
	 */
	public void meBounce(OceanObject obj2) {

		if (getPositionX() > obj2.getPositionX()) {

			setDirectionX(abs(getDirectionX()));
			setxDirConst(abs(getxDirConst()));

		} else {
			setDirectionX(abs(getDirectionX()) * (-1));
			setxDirConst(abs(getxDirConst()) * (-1));
		}

		int fall = collidesWithOceanBounds();

		if (fall != KEINE_KOLLISION) {
			updateDirectionWhenCollidingWithOceanBounds(fall);
		}

	}

	/**
	 * The move method adds the adds the new direction the the current
	 * position (move) and updates the ocean objects image.
	 *
	 * @return an integer specifying whether or not the object has collided with the ocean's bounds
	 */
	public synchronized int move() {

		int[] newPosition = new int[2];
		newPosition[0] = getPositionX() + getDirectionX();
		newPosition[1] = getPositionY() + getDirectionY();
		setPosition(newPosition);
		int fall = collidesWithOceanBounds();

		if (fall != 0) {

			updateDirectionWhenCollidingWithOceanBounds(fall);

		} else {

			setDirectionX((int) (getxDirConst() + (getxDirVar() * random())));
			setDirectionY((int) (getyDirConst() + (getyDirVar() * random())));
		}

		updateImageWhenMoving();
		setAge(getAge() + 1);
		return fall;
	}

	/**
	 * Movement collision two.
	 *
	 * @param obj2 the obj2
	 * @return the boolean
	 */
	public synchronized Boolean movementCollisionTwo(OceanObject obj2) {
		return RectangleIntersect(obj2);
	}

	/**
	 * This method describes the reaction of an object with a second object.
	 * This method is override in every child class.
	 * @param obj2 is the second object in the collision
	 */
	public abstract void reaktion(OceanObject obj2);

	/**
	 * This method detects a collision of two objects.
	 *
	 * @param obj2 is the second object in the collision
	 * @return true if and only if a collision has occurred
	 */
	public synchronized Boolean RectangleIntersect(OceanObject obj2) {

		// obj1-Infos
		int obj1PosX = getPositionX();
		int obj1PosY = getPositionY();

		int obj1Width = getIcon().getIconWidth();
		obj1Width += abs(getDirectionX());
		int obj1Height = getIcon().getIconHeight();
		obj1Height += abs(getDirectionY());

		int obj1CenterX = obj1PosX + (obj1Width / 2);
		int obj1CenterY = obj1PosY + (obj1Height / 2);

		// obj2-Infos
		int obj2PosX = obj2.getPositionX();
		int obj2PosY = obj2.getPositionY();

		int obj2Width = obj2.getIcon().getIconWidth();
		obj2Width += abs(obj2.getDirectionX());

		int obj2Height = obj2.getIcon().getIconHeight();
		obj2Height += abs(obj2.getDirectionY());

		int obj2CenterX = obj2PosX + (obj2Width / 2);
		int obj2CenterY = obj2PosY + (obj2Height / 2);

		// xDistance of obj1, obj2
		int xDistanceObj1Obj2 = abs(obj2CenterX - obj1CenterX);
		int yDistanceObj1Obj2 = abs(obj2CenterY - obj1CenterY);

		return

				(xDistanceObj1Obj2 <= ((obj1Width / 2) + (obj2Width / 2)))
				&& (yDistanceObj1Obj2 <= ((obj2Height / 2) + (obj2Height / 2)));

	}

	/**
	 * Sets the age of the oceanObject.
	 *
	 * @param age the age to set
	 */
	public void setAge(int age) {
		this.age = age;
	}

	/**
	 * Sets the direction of the oceanObject.
	 *
	 * @param direction the direction to set
	 */
	public void setDirection(int[] direction) {
		this.direction = direction;
	}

	/**
	 * Sets the object's id.
	 *
	 * @param ID the iD to set
	 */
	public void setID(int ID) {
		this.ID = ID;
	}

	/**
	 * Sets the maximum age.
	 *
	 * @param maximumAge the maximumAge to set
	 */
	public void setMaximumAge(int maximumAge) {
		this.maximumAge = maximumAge;
	}

	/**
	 * Sets the ocean.
	 *
	 * @param myOcean the ocean to set
	 */
	public void setMyOcean(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the ocean object's name.
	 *
	 * @param name the new name
	 */
	public void setOceanObjectName(String name) {
		this.oceanObjectName = name;
	}

	/**
	 * Sets the object's position.
	 *
	 * @param position the position to set
	 */
	public void setPosition(int[] position) {
		this.position = position;
	}

	/**
	 * Sets the object's position's x value.
	 *
	 * @param positionX the new position's x value
	 */
	public void setPositionX(int positionX) {
		this.position[0] = positionX;
	}

	/**
	 * Sets the object's position's y value.
	 *
	 * @param positionY position's y value.
	 */
	public void setPositionY(int positionY) {
		this.position[1] = positionY;
	}

	/**
	 * Generate a new random number generator.
	 *
	 * @param rand the random number generator to set
	 */
	public void setRand(Random rand) {
		this.rand = rand;
	}

	/**
	 * Sets the object's tagged attribute.
	 *
	 * @param tagged whether or not the object is tagged
	 */
	public void setTagged(Boolean tagged) {
		this.tagged = tagged;
		updateImageWhenMoving();
	}

	/**
	 * Sets the object's toppled-left attribute.
	 *
	 * @param toppledLeft whether or not the object is toppled left
	 */
	public void setToppledLeft(Boolean toppledLeft) {
		this.toppledLeft = toppledLeft;
	}

	/**
	 * Sets the object's toppled-right attribute.
	 *
	 * @param toppledRight whether or not the object is toppled right
	 */
	public void setToppledRight(Boolean toppledRight) {
		this.toppledRight = toppledRight;
	}

	/**
	 * Sets the objects type attribute.
	 *
	 * @param type the type to set
	 */
	public void setType(int type) {
		this.type = type;
	}

	/**
	 * Sets the object's upright attribute.
	 *
	 * @param upright whether or not the object is upright
	 */
	public void setUpright(Boolean upright) {
		this.upright = upright;
	}

	/**
	 * Sets the x-dir-const value.
	 *
	 * @param xDirConst  the xDirConst to set
	 */
	public void setxDirConst(int xDirConst) {
		this.xDirConst = xDirConst;
	}

	/**
	 * Sets the x-dir-var value.
	 *
	 * @param xDirVar the xDirVar to set
	 */
	public void setxDirVar(int xDirVar) {
		this.xDirVar = xDirVar;
	}

	/**
	 * Sets the y-dir-const value.
	 *
	 * @param yDirConst the yDirConst to set
	 */
	public void setyDirConst(int yDirConst) {
		this.yDirConst = yDirConst;
	}

	/**
	 * Sets the y-dir-var value.
	 *
	 * @param yDirVar the yDirVar to set
	 */
	public void setyDirVar(int yDirVar) {
		this.yDirVar = yDirVar;
	}

	/**
	 * Sets the direction's x value.
	 *
	 * @param directionX the new direction's x value
	 */
	public void setDirectionX(int directionX) {
		direction[0] = directionX;
	}

	/**
	 * Sets the direction's y value.
	 *
	 * @param directionY the new direction's y value
	 */
	public void setDirectionY(int directionY) {
		direction[1] = directionY;
	}

	/**
	 * Sets the hunger of the oceanObject.
	 *
	 * @param hunger  the hunger to set
	 */
	public void setHunger(int hunger) {
		this.hunger = hunger;
	}

	/**
	 * Sets the objects icon.
	 *
	 * @param icon the icon to set
	 */
	public void setIcon(ImageIcon icon) {
		this.icon = icon;
	}

	/**
	 * Sets the object's icon.
	 *
	 * @param filename the new icon's file name
	 */
	public void setIcon(String filename) {

		try {
			ImageIcon newIcon = new ImageIcon(getClass().getResource(
					"images/" + filename));
			setIcon(newIcon);
		} catch (Exception e) {
			System.err.println("Couldn't load image " + filename);
		}
	}

	/**
	 * Topples the object left or right, by collision with object2.
	 *
	 * @param obj2 the object this object collides with
	 */
	public void topple(OceanObject obj2) {

		if (getPositionX() > obj2.getPositionX()) {

			if (obj2.getUpright()) {

				obj2.toppleLeft();
			}

		} else {

			if (obj2.getUpright()) {

				obj2.toppleRight();
			}
		}
	}

	/**
	 * Topples the object left.
	 */
	public void toppleLeft() {

		setToppledLeft(true);
		setUpright(false);
		setToppledRight(false);
		setPosition(new int[] { getPositionX(), getPositionY() + 40 });

		if (getTagged()) {
			setIcon("cokecan-left-highlighted.png");
		} else {
			setIcon("cokecan-left.png");
		}

	}

	/**
	 * Topples the object right.
	 */
	public void toppleRight() {

		setToppledRight(true);
		setUpright(false);
		setToppledLeft(false);
		setPosition(new int[] { getPositionX(), getPositionY() + 40 });

		if (getTagged()) {
			setIcon("cokecan-right-highlighted.png");
		} else {
			setIcon("cokecan-right.png");

		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {

		String className = getClass().getCanonicalName().replace("model.", "");
		String posString = getPositionString();

		String result = className + "\t" + "id: " + getID() + "\t\t";
		result += "name: " + getOceanObjectName() + "\t";
		result += "pos: " + posString
				+ (posString.length() > 10 ? "\t" : "\t\t");
		result += "dir: " + getDirectionString() + "\t\t";
		result += "age:" + getAge() + "\t";
		result += "max. age:" + getMaximumAge() + "\t";

		result += "\n";
		return result;
	}

	/**
	 * Updates the object's direction when colliding with ocean bounds.
	 * This method is overridden in every child class.
	 *
	 * @param collision an integer specifying which bound, if any, the object has collided with
	 */
	protected abstract void updateDirectionWhenCollidingWithOceanBounds(
			int collision);

	/**
	 * Updates the image when moving.
	 */
	private void updateImageWhenMoving() {

		// schwimmt nach rechts oder steht still
		if (getDirectionX() >= 0) {

			if (getTagged()) { // Object is tagged

				switch (getType()) {

				case FISH:
					setIcon("fish-right-highlighted.png");
					break;

				case SHARK:
					setIcon("shark-right-highlighted.png");
					break;

				case STONE:
					setIcon("stone-highlighted.png");
					break;

				case PLANT:
					setIcon("plant-highlighted.png");
					break;

				case COKE_CAN:

					if (getToppledLeft()) {
						setIcon("cokecan-left-highlighted.png");
					}
					if (getToppledRight()) {
						setIcon("cokecan-right-highlighted.png");
					}
					if (getUpright()) {
						setIcon("cokecan-highlighted.png");
					}
					break;

				}

			} else { // Object schwimmt nach rechts oder steht still und ist
				// nicht tagged
				switch (getType()) {

				case FISH:
					setIcon("fish.png");
					break;

				case SHARK:
					setIcon("shark.png");
					break;

				case STONE:
					setIcon("stone.png");
					break;

				case PLANT:
					setIcon("plant.png");
					break;

				case COKE_CAN:

					if (getToppledLeft()) {
						setIcon("cokecan-left.png");
					}
					if (getToppledRight()) {
						setIcon("cokecan-right.png");
					}
					if (getUpright()) {
						setIcon("cokecan.png");
					}
					break;
				}
			}

		} else { // object bewegt sich nach links

			if (getTagged()) { // Object is tagged

				switch (getType()) {

				case FISH:
					setIcon("fish-highlighted.png");
					break;

				case SHARK:
					setIcon("shark-left-highlighted.png");
					break;

				}

			} else { // Object ist nicht tagged
				switch (getType()) {

				case FISH:
					setIcon("fish-left.png");
					break;

				case SHARK:
					setIcon("shark-left.png");
					break;

				}
			}
		}
	}

	/**
	 * You bounce.
	 * This method lets the second object bounce from this object.
	 * @param obj2 the second object
	 */
	public void youBounce(OceanObject obj2) {

		if (getPositionX() > obj2.getPositionX()) {

			obj2.setDirectionX(abs(obj2.getDirectionX()) * (-1));
			obj2.setxDirConst(abs(obj2.getxDirConst()) * (-1));
		} else {

			obj2.setDirectionX(abs(obj2.getDirectionX()));
			obj2.setxDirConst(abs(obj2.getxDirConst()));
		}

		int fall = collidesWithOceanBounds();

		if (fall != KEINE_KOLLISION) {
			updateDirectionWhenCollidingWithOceanBounds(fall);

		}
	}
}