package model;

import static java.lang.Math.abs;
import static model.Ocean.COKE_CAN;
import static model.Ocean.FISH;
import static model.Ocean.PLANT;
import static model.Ocean.SHARK;
import static model.Ocean.STONE;

/**
 * The Class Shark.
 */
public class Shark extends OceanObject {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 2636101562741328192L;

	/**
	 * Instantiates a new shark.
	 * 
	 * @param ID  the id
	 * @param name the name
	 * @param position the position
	 * @param ocean the ocean
	 * @param maximumAge the maximum age
	 * @throws Exception the exception
	 */
	public Shark(int ID, String name, int[] position, OceanInterface ocean,
			int maximumAge) throws Exception {

		super(ID, name, position, ocean, maximumAge);

		setType(SHARK);
		setxDirConst(11);
		setyDirConst(-5);
		setxDirVar(-5);
		setyDirVar(-4);
		setHunger(10);
	}

	// _______________________________________________________________________________________________________

	/**
	 * This method describes the way an object reacts after a collision.
	 * 
	 * @param obj2 the second one of the collision
	 */
	@Override
	public void reaktion(OceanObject obj2) {

		switch (obj2.getType()) {

		case FISH:

			int r = abs(getRand().nextInt()) % 5;

			if (r == 0) {

				getMyOcean().removeOceanObject(obj2);
				setMaximumAge((int) (getMaximumAge() * 1.1));
			}

			youBounce(obj2);
			break;

		case SHARK:

			if (getMyOcean().getNumberOfOceanObjectsByType(SHARK) <= 4) {
				if (getAge() >= ((getMaximumAge() * 1) / 3)) {

					int sharks = getMyOcean().getNumberOfOceanObjectsByType(
							SHARK);

					spawnRandomShark(500 - (7 * sharks));
					obj2.setAge((getMaximumAge() * 96) / 100);
				}
			}
			break;

		case STONE:
			meBounce(obj2);
			break;

		case COKE_CAN:
			meBounce(obj2);
			topple(obj2);
			break;

		case PLANT:
			meBounce(obj2);
			setAge((int) (getAge() * 0.8));
			break;
		}
	}

	/**
	 * Spawn random shark.
	 * 
	 * @param maximumAge
	 *            the maximum age
	 */
	private void spawnRandomShark(int maximumAge) {

		int[] spawnPosition = new int[2];

		spawnPosition[0] = getRand().nextInt(
				(int) (((getMyOcean().getWidth()) - getDimensionsOfIcon()
						.getWidth()) + 1));

		spawnPosition[1] = getRand().nextInt(
				(int) (((getMyOcean().getDepth()) - getDimensionsOfIcon()
						.getHeight()) + 1));

		try {
			new Shark(Ocean.makeOOID(), "Sharky", spawnPosition, getMyOcean(),
					getMaximumAge());
		} catch (Exception e) {
		}
	}

	/**
	 * Describes the way an objects reacts with the oceans bounds.
	 * 
	 * @param collisionDetected is a collision detected
	 * 
	 */
	@Override
	protected void updateDirectionWhenCollidingWithOceanBounds(
			int collisionDetected) {

		switch (collisionDetected) {

		case KOLLISION_RECHTS:
			setxDirConst(-abs(getxDirConst()));
			setxDirVar(-getxDirVar());

			setDirectionX(-getDirectionX());
			setDirectionY(-getDirectionY());
			setPositionX(getMyOcean().getWidth() - getDimensionsOfIcon().width);
			break;

		case KOLLISION_OBEN:
			setDirection(new int[] { getDirectionX(), -getDirectionY() });
			setyDirConst(-getyDirConst());
			setyDirVar(-getyDirVar());

			setPositionY(0);
			break;

		case KOLLISION_UNTEN:
			setDirection(new int[] { getDirectionX(), -getDirectionY() });
			setyDirConst(-getyDirConst());
			setyDirVar(-getyDirVar());
			break;

		case KOLLISION_LINKS:
			setxDirConst(-getxDirConst());
			setxDirVar(-getxDirVar());

			setDirectionX(-getDirectionX());
			setDirectionY(-getDirectionY());

			setPositionX(0);
			break;

		case KEINE_KOLLISION:
			break;

		default:
			break;

		}
	}
}