package stuff;

/**
 *  This class is a generic Pair-Class, that is the most awesome thing in our code.
 *  It makes sure that each component is of the specified type so that no casts are necessary when using them.
 *  We wrote the Pair class to be able to deal with pairs of any type, because similar classes,
 *  such as java.awt.Dimension, only cater for pairs of integers or other special cases.
 *  It might also be useful to pair objects of different types.
 *
 * @param <T> the generic type of the pair's first component
 * @param <S> the generic type of the pair's second component
 */
public class Pair<T, S> {
	private T first;
	private S second;

	// ____________________________________________________________________________________
	/**
	 * Constructs a pair.
	 *
	 * @param first the pair's first component
	 * @param second first the pair's first component
	 */
	public Pair(T first, S second) {

		setFirst(first);
		setSecond(second);
	}

	// ____________________________________________________________________________________
	/**
	 * Returns the pair's first component.
	 *
	 * @return first the pair's first component
	 */
	public T getFirst() {
		return first;
	}

	/**
	 * Returns the pair's second component.
	 *
	 * @return second the pair's second component
	 */
	public S getSecond() {
		return second;
	}

	/**
	 * Sets the pair's first component.
	 *
	 * @param first the pair's first component
	 */
	public void setFirst(T first) {
		this.first = first;
	}

	/**
	 * Sets the pair's second component.
	 *
	 * @param second the pair's second component
	 */
	public void setSecond(S second) {
		this.second = second;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {

		String result = "";
		result += "(";
		result += getFirst();
		result += ", ";
		result += getSecond();
		result += ")";
		return result;
	}
}
