package view;

import java.awt.FlowLayout;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * The Class ButtonPanel that contains all buttons and other input fields.
 */
public class ButtonPanel extends JPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The about button. */
	private JButton aboutButton = new JButton("about");

	/** The add button. */
	private final JButton addButton = new JButton("Add");

	/** The clear button. */
	private JButton clearButton = new JButton("Clear");

	/** The load button. */
	private final JButton loadButton = new JButton("Load");

	/** The maximum age field. */
	private JTextField maximumAgeField = new JTextField(5);

	/** The maximum age label. */
	private JLabel maximumAgeLabel = new JLabel("max. Alter:");

	/** The name field. */
	private JTextField nameField = new JTextField(15);

	/** The name label. */
	private JLabel nameLabel = new JLabel("Name:");

	/** The ocean object types, as String. */
	private final JComboBox<String> oceanObjectTypes = new JComboBox<String>(
			new String[] { "Fish", "Shark", "Stone", "Bubble", "Plant",
			"Coke Can" });

	/** The print button. */
	private JButton printButton = new JButton("Print");

	/** The rem button. */
	private final JButton remButton = new JButton("Remove");

	/** The save button. */
	private final JButton saveButton = new JButton("Save");

	/** The start stop button. */
	private final JButton startStopButton = new JButton("Start");

	/** The step button. */
	private JButton stepButton = new JButton("Step");

	/** The x label. */
	private JLabel xLabel = new JLabel("x-Pos");

	/** The x value field. */
	private JTextField xValueField = new JTextField(3);

	/** The y label. */
	private JLabel yLabel = new JLabel("y-Pos");

	/** The y value field. */
	private JTextField yValueField = new JTextField(3);

	// ________________________________________________________________

	/**
	 * Instantiates a new button panel.
	 */
	protected ButtonPanel() {
		super(false);
		setVisible(false);
		setLayout(new FlowLayout());
		addComponentsToButtonPanel();
		setToolTips();
		setVisible(true);
	}

	/**
	 * Adds the components to button panel.
	 */
	private void addComponentsToButtonPanel() {

		add(oceanObjectTypes);

		add(getNameLabel());
		add(getNameField());

		add(getMaximumAgeLabel());
		add(getMaximumAgeField());

		add(getxLabel());
		add(getXValueField());
		add(getyLabel());
		add(getYValueField());

		add(getAddButton());
		add(getRemButton());

		add(getStartStopButton());
		add(getStepButton());

		add(getPrintButton());

		add(getSaveButton());
		add(getLoadButton());
		add(getClearButton());
		add(getAboutButton());

	}

	/**
	 * Adds the listeners.
	 * 
	 * @param aL
	 *            the ActionListener that listens to all the buttons
	 */
	public void addListeners(ActionListener aL) {

		getAddButton().addActionListener(aL);
		getRemButton().addActionListener(aL);
		getStartStopButton().addActionListener(aL);
		getStepButton().addActionListener(aL);
		getPrintButton().addActionListener(aL);
		getSaveButton().addActionListener(aL);
		getLoadButton().addActionListener(aL);
		getClearButton().addActionListener(aL);
		getAboutButton().addActionListener(aL);
	}

	/**
	 * Gets the about button.
	 * 
	 * @return the aboutButton
	 */
	private JButton getAboutButton() {
		return aboutButton;
	}

	/**
	 * Gets the adds the button.
	 * 
	 * @return addButton
	 */
	public JButton getAddButton() {
		return this.addButton;
	}

	/**
	 * Gets the clear button.
	 * 
	 * @return the clearButton
	 */
	private JButton getClearButton() {
		return clearButton;
	}

	/**
	 * Gets the load button.
	 * 
	 * @return loadButton
	 */
	private JButton getLoadButton() {

		return this.loadButton;
	}

	/**
	 * Gets the maximum age field.
	 * 
	 * @return the maximumAgeField
	 */
	public JTextField getMaximumAgeField() {
		return maximumAgeField;
	}

	/**
	 * Gets the maximum age label.
	 * 
	 * @return the maximumAgeLabel
	 */
	public JLabel getMaximumAgeLabel() {
		return maximumAgeLabel;
	}

	/**
	 * Gets the name field.
	 * 
	 * @return nameField
	 */
	public JTextField getNameField() {
		return nameField;
	}

	/**
	 * Gets the name for new object.
	 * 
	 * @return the name for new object
	 */
	public String getNameForNewObject() {

		return nameField.getText();

	}

	/**
	 * Gets the name label.
	 * 
	 * @return the nameLabel
	 */
	protected JLabel getNameLabel() {
		return nameLabel;
	}

	/**
	 * Gets the ocean object types string.
	 * 
	 * @return oceanObjectTypes as String
	 */
	public String getOceanObjectTypesString() {

		return (String) this.oceanObjectTypes.getSelectedItem();

	}

	/**
	 * Gets the prints the button.
	 * 
	 * @return the printButton
	 */
	private JButton getPrintButton() {
		return printButton;
	}

	/**
	 * Gets the rem button.
	 * 
	 * @return remButton
	 */
	private JButton getRemButton() {

		return this.remButton;
	}

	/**
	 * Gets the save button.
	 * 
	 * @return saveButton
	 */
	private JButton getSaveButton() {

		return this.saveButton;
	}

	/**
	 * Gets the start stop button.
	 * 
	 * @return startButton
	 */
	public JButton getStartStopButton() {

		return this.startStopButton;
	}

	/**
	 * Gets the step button.
	 * 
	 * @return the stepButton
	 */
	public JButton getStepButton() {
		return stepButton;
	}

	/**
	 * Gets the x label.
	 * 
	 * @return the xLabel
	 */
	private JLabel getxLabel() {
		return xLabel;
	}

	/**
	 * Gets the x value field.
	 * 
	 * @return xValue form xValueField
	 */
	public JTextField getXValueField() {

		return this.xValueField;
	}

	// _______________________________________________________
	/**
	 * Gets the y label.
	 * 
	 * @return the yLabel
	 */
	private JLabel getyLabel() {
		return yLabel;
	}

	/**
	 * Gets the y value field.
	 * 
	 * @return yValue form xValueField
	 */
	public JTextField getYValueField() {

		return this.yValueField;
	}

	/**
	 * Sets the about button.
	 * 
	 * @param aboutButton
	 *            the aboutButton to set
	 */
	protected void setAboutButton(JButton aboutButton) {
		this.aboutButton = aboutButton;
	}

	/**
	 * Sets the clear button.
	 * 
	 * @param clearButton
	 *            the clearButton to set
	 */
	protected void setClearButton(JButton clearButton) {
		this.clearButton = clearButton;
	}

	/**
	 * Sets the maximum age field.
	 * 
	 * @param maximumAgeField
	 *            the maximumAgeField to set
	 */
	public void setMaximumAgeField(JTextField maximumAgeField) {
		this.maximumAgeField = maximumAgeField;
	}

	/**
	 * Sets the maximum age label.
	 * 
	 * @param maximumAgeLabel
	 *            the maximumAgeLabel to set
	 */
	public void setMaximumAgeLabel(JLabel maximumAgeLabel) {
		this.maximumAgeLabel = maximumAgeLabel;
	}

	/**
	 * Sets the name field.
	 * 
	 * @param nameField
	 *            the new name field
	 */
	public void setNameField(JTextField nameField) {
		this.nameField = nameField;
	}

	/**
	 * Sets the name label.
	 * 
	 * @param nameLabel
	 *            the nameLabel to set
	 */
	public void setNameLabel(JLabel nameLabel) {
		this.nameLabel = nameLabel;
	}

	/**
	 * Sets the prints the button.
	 * 
	 * @param printButton
	 *            the printButton to set
	 */
	protected void setPrintButton(JButton printButton) {
		this.printButton = printButton;
	}

	/**
	 * Sets the step button.
	 * 
	 * @param stepButton
	 *            the stepButton to set
	 */
	protected void setStepButton(JButton stepButton) {
		this.stepButton = stepButton;
	}

	/**
	 * Sets the tool tips.
	 */
	private void setToolTips() {

		getAddButton().setToolTipText("Add an object");
		getRemButton().setToolTipText("Remove an object");
		getStartStopButton().setToolTipText("Start or stop the simulation");
		getStepButton().setToolTipText("Let every object make a single move");
		getPrintButton().setToolTipText(
				"Print information about the ocean and includet objects.");
		getSaveButton().setToolTipText("Save the ocean and all its objects");
		getLoadButton().setToolTipText("Load an saved Ocean");
		getClearButton().setToolTipText("Clear the ocean");

		getNameField().setToolTipText("Set a name for the ocean object");
		getMaximumAgeField().setToolTipText(
				"Set a maximum age.\n" + "z.B. \n" + "Fish: 100\n"
						+ "Shark: 400 \n" + "XShark: 9000\n" + "Stone: 10000"
						+ "Plant: 500" + "Bubble: 100" + "CokeCan: 9999");
	}

	/**
	 * Sets the x label.
	 * 
	 * @param xLabel
	 *            the xLabel to set
	 */
	protected void setxLabel(JLabel xLabel) {
		this.xLabel = xLabel;
	}

	/**
	 * Sets the x value field.
	 * 
	 * @param xValueField
	 *            the xValueField to set
	 */
	protected void setxValueField(JTextField xValueField) {
		this.xValueField = xValueField;
	}

	/**
	 * Sets the y label.
	 * 
	 * @param yLabel
	 *            the yLabel to set
	 */
	protected void setyLabel(JLabel yLabel) {
		this.yLabel = yLabel;
	}

	/**
	 * Sets the y value field.
	 * 
	 * @param yValueField
	 *            the yValueField to set
	 */
	protected void setyValueField(JTextField yValueField) {
		this.yValueField = yValueField;
	}
}
