package view;

import java.awt.BorderLayout;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelListener;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import model.OceanInterface;

/**
 * The Class OceanGUI. Makes a frame that contains a water panel, a button panel
 * and a status panel.
 */
public class OceanGUI extends JFrame implements OceanGUIInterface {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 2532578208638497123L;

	/** The button panel. */
	private ButtonPanel buttonPanel;

	/** The ocean. */
	private OceanInterface myOcean;

	/** The status panel. */
	private StatusPanel statusPanel;

	/** The water panel. */
	private WaterPanel waterPanel;

	// ___________________________________________________________________________________________________

	/**
	 * Instantiates a new oceanGUI.
	 * 
	 * @param title
	 *            the title
	 * @param model
	 *            the model
	 */
	public OceanGUI(String title, OceanInterface model) {

		super(title);
		setVisible(false);
		setMyOcean(model);
		if (getMyOcean() == null) { // load GUI without model
			System.err.println("GUI hat keinen Ozean");
			System.exit(3);
		}
		setButtonPanel(new ButtonPanel());
		setWaterPanel(new WaterPanel(myOcean));
		setStatusPanel(new StatusPanel(myOcean));
		int depth = 90 + myOcean.getDepth();
		int width = myOcean.getWidth();
		setResizable(false);
		setLayout(new BorderLayout(0, 2));
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		add(buttonPanel, BorderLayout.NORTH);
		add(waterPanel, BorderLayout.CENTER);
		add(statusPanel, BorderLayout.SOUTH);
		setSize(width, depth);
		setVisible(true);
		Thread t2 = new Thread(statusPanel);
		t2.start();
	}

	// ___________________________________________________________________________________________________

	/**
	 * Gets the button panel.
	 * 
	 * @return buttonPanel
	 */
	@Override
	public ButtonPanel getButtonPanel() {
		return buttonPanel;
	}

	/**
	 * Gets the my ocean.
	 * 
	 * @return the myOcean
	 */
	@Override
	public OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * Gets the status panel.
	 * 
	 * @return statusPanel
	 */
	@Override
	public StatusPanel getStatusPanel() {
		return statusPanel;
	}

	/**
	 * Gets the water panel.
	 * 
	 * @return waterPanel
	 */
	@Override
	public WaterPanel getWaterPanel() {
		return waterPanel;
	}

	/**
	 * Sets the button panel.
	 * 
	 * @param buttonPanel
	 *            the new button panel
	 */
	@Override
	public void setButtonPanel(ButtonPanel buttonPanel) {
		this.buttonPanel = buttonPanel;
	}

	/**
	 * Sets the ocean.
	 * 
	 * @param myOcean
	 *            the myOcean to set
	 */
	@Override
	public void setMyOcean(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the status panel.
	 * 
	 * @param statusPanel
	 *            the new status panel
	 */
	@Override
	public void setStatusPanel(StatusPanel statusPanel) {
		this.statusPanel = statusPanel;
	}

	/**
	 * Sets the water panel.
	 * 
	 * @param waterPanel
	 *            the new water panel
	 */
	@Override
	public void setWaterPanel(WaterPanel waterPanel) {
		this.waterPanel = waterPanel;
	}

	/**
	 * Add the mouse listeners.
	 * 
	 * @param mouseController
	 *            the mouse listener
	 * @param mouseController2
	 *            the mouse wheel listener2
	 */
	@Override
	public void addMouseListeners(MouseListener mouseController,
			MouseWheelListener mouseController2) {

		getWaterPanel().addMouseListener(mouseController);
		getWaterPanel().addMouseWheelListener(mouseController2);
	}

	@Override
	public void showErrorMessage(String errorMsg) {
		JFrame frame = new JFrame();
		JOptionPane.showMessageDialog(frame, errorMsg, "OceanLife",
				JOptionPane.ERROR_MESSAGE);
	}
}
