package view;

import java.awt.GridLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;

import model.OceanInterface;
import stuff.Pair;

/**
 * The Class StatusPanel displays infos (population, time, info) about the
 * oceans objects.
 */
public class StatusPanel extends JPanel implements Runnable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -7044599228727393365L;

	/** The my ocean. */
	private OceanInterface myOcean = null;

	/** The population label. */
	private JLabel populationLabel = new JLabel("population: ?");

	/** The status label. */
	private final JLabel statusLabel = new JLabel(
			"Freuen Sie sich auf Version 2! (Heute in einer Woche)");

	/** The time elapsed label. */
	private JLabel timeElapsedLabel = new JLabel("time: ?");

	// __________________________________________________________________________________

	/**
	 * Instantiates a new status panel.
	 * 
	 * @param ocean
	 *            the ocean
	 */
	protected StatusPanel(OceanInterface ocean) {

		setMyOcean(ocean);
		setLayout(new GridLayout(1, 3));
		addComponentsToFrame();
		initStatusPanel();
		setVisible(true);
	}
	// __________________________________________________________________________________


	/**
	 * Gets the ocean.
	 * 
	 * @return the myOcean
	 */
	private OceanInterface getMyOcean() {
		return myOcean;
	}

	/**
	 * Gets the population label.
	 * 
	 * @return the populationLabel
	 */
	private JLabel getPopulationLabel() {
		return populationLabel;
	}

	/**
	 * Gets the status label.
	 * 
	 * @return statusLabel
	 */
	private JLabel getStatusLabel() {

		return this.statusLabel;
	}

	/**
	 * Gets the time elapsed label.
	 * 
	 * @return the timeElapsedLabel
	 */
	private JLabel getTimeElapsedLabel() {
		return timeElapsedLabel;
	}



	/**
	 * Sets the ocean.
	 * 
	 * @param myOcean
	 *            the ocean to set
	 */
	private void setMyOcean(OceanInterface myOcean) {
		this.myOcean = myOcean;
	}

	/**
	 * Sets the population label.
	 * 
	 * @param popLabel
	 *            the new population label
	 */
	protected void setPopulationLabel(JLabel popLabel) {
		this.populationLabel = popLabel;
	}

	/**
	 * Sets the population label text.
	 * 
	 * @param text
	 *            the new population label text
	 */
	private void setPopulationLabelText(String text) {

		populationLabel.setText(text);

	}

	/**
	 * Set the status label text.
	 * 
	 * @param text
	 *            the new status label text
	 */
	public void setStatusLabelText(String text) {
		statusLabel.setText(text);
	}

	/**
	 * Sets the time elapsed label.
	 * 
	 * @param timeLabel
	 *            the new time elapsed label
	 */
	protected void setTimeElapsedLabel(JLabel timeLabel) {
		this.timeElapsedLabel = timeLabel;
	}

	/**
	 * Sets the time elapsed label text.
	 * 
	 * @param text
	 *            the new time elapsed label text
	 */
	private void setTimeElapsedLabelText(String text) {

		timeElapsedLabel.setText(text);

	}

	/**
	 * Sleep.
	 */
	private void sleep() {
		try {
			Thread.sleep(20);
		} catch (InterruptedException e) {
			System.out.println("StatusPanel: I can't get no sleep...");
		}
	}

	/**
	 * Do nothing.
	 */
	private void doNothing() {

		try {
			Thread.sleep(150);
		} catch (InterruptedException e1) {
			System.err.println("StatusPanel: I can't get no sleep...");
		}
	}

	/**
	 * Adds the components to frame.
	 */
	private void addComponentsToFrame() {

		getPopulationLabel().setHorizontalAlignment(JLabel.CENTER);
		add(getPopulationLabel());
		populationLabel.setVisible(true);
		getStatusLabel().setHorizontalAlignment(JLabel.CENTER);
		add(getStatusLabel());
		statusLabel.setVisible(true);
		getTimeElapsedLabel().setHorizontalAlignment(JLabel.CENTER);
		add(getTimeElapsedLabel());
		timeElapsedLabel.setVisible(true);
	}

	/**
	 * Initialises the status panel.
	 */
	private void initStatusPanel() {
		update();
	}

	@Override
	public void run() {

		while (true) {

			if (getMyOcean().isOceanRunning()) {

				this.update();
				this.repaint();
				sleep();

			} else {
				doNothing();
			}
		}
	}

	/**
	 * This method gets the info from the ocean and displays it.
	 */
	public void update() {
		Pair<Integer, Integer> info = getMyOcean().getStatusPanelInfo();

		String timeElapsedLabelText = "";
		timeElapsedLabelText += "time: ";
		timeElapsedLabelText += info.getFirst().toString();

		setTimeElapsedLabelText(timeElapsedLabelText);

		String populationLabelText = "";
		populationLabelText += "population: ";
		populationLabelText += info.getSecond().toString();

		setPopulationLabelText(populationLabelText);
	}
}
